﻿using System;
using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.MetaOptimization {
  public delegate void MutateDelegate(IRandom random, IOptimizable configuartion, IIntValueManipulator intValueManipulator, IDoubleValueManipulator doubleValueManipulator);
  public delegate void CrossDelegate(IRandom random, IOptimizable configuartion, IOptimizable other, IIntValueCrossover intValueCrossover, IDoubleValueCrossover doubleValueCrossover);

  public interface IOptimizable : IItem {
    bool IsOptimizable { get; }
    bool Optimize { get; set; }
    ConstrainedValue ActualValue { get; set; }
    
    void Randomize(IRandom random);
    void Mutate(IRandom random, MutateDelegate mutate, IIntValueManipulator intValueManipulator, IDoubleValueManipulator doubleValueManipulator);
    void Cross(IRandom random, IOptimizable other, CrossDelegate cross, IIntValueCrossover intValueCrossover, IDoubleValueCrossover doubleValueCrossover);
    string ParameterInfoString { get; }
    void CollectOptimizedParameterNames(List<string> parameterNames, string prefix);
    double CalculateSimilarity(IOptimizable optimizable);

    /// <summary>
    /// Recursively gets all optimizables which have Optimize=true
    /// </summary>
    List<IOptimizable> GetAllOptimizables();

    event EventHandler IsOptimizableChanged;
    event EventHandler OptimizeChanged;

    /// <summary>
    /// indicates if the actualvalues are modifiable
    /// </summary>
    bool ValuesReadOnly { get; set; }
  }
}
